<?php
/**
 * Plugin Name:         RS SEO Basics
 * Description:         The most minimal SEO plugin possible.
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-seo-basics/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-seo-basics
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             0.1.7
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

include __DIR__ . '/trait-opengraph.php';
include __DIR__ . '/trait-schema.php';

add_action( 'plugins_loaded', function() {
    class SEO_Basics extends PluginBase {
        protected $plugin_file = __FILE__;

        use SEO_OpenGraph;
        use SEO_Schema;

        /************************************************************************
         *
         *	Open Graph and Schema headers
         *
         ************************************************************************/

        protected $wp_head_callback_priority = 1;
        function do_wp_head_action() {
            if ( is_admin() )
                return;

            echo "\n\n\t<!-- START -- RS SEO basics -->\n";

            $graph = $this->opengraph();
            $link_rel = $this->link_rel();

            // Why don't WordPress do this automatically?
            if ( !empty( $graph['og:description'] ) )
                $this->opengraph_print_meta( [ 'name' => "description", 'content' => $graph['og:description'] ] );

            // <link rel=xx href=yy>
            foreach ( $link_rel as $key => $value )
                if ( !empty( $value ) )
                    $this->opengraph_print_meta( [ 'rel' => $key, 'href' => $value ], 'link' );

            // <meta property=xx content=yy>
            foreach ( $graph as $key => $value ) {
                if ( !empty( $value ) ) {
                    if ( ! is_array( $value ) )
                        $value = [ $value ];
                    foreach ( $value as $v )
                        $this->opengraph_print_meta( [ 'property' => "$key", 'content' => $v ] );
                }
            }

            $schema = $this->schema_org();
            printf( "<script type=\"application/ld+json\">\n%s\n</script>\n", json_encode( $schema, JSON_PRETTY_PRINT ) );


            echo "\t<!-- END -- RS SEO basics -->\n\n";
        }

        /************************************************************************
         *
         *	Options
         *
         ************************************************************************/

        protected function get_facebook_url() {
            return $this->get_matching_sameAs_url( 'https://www.facebook.com/' );
        }

        protected function get_twitter_url() {
            return $this->get_matching_sameAs_url( 'https://twitter.com/' )
                ?? $this->get_matching_sameAs_url( 'https://x.com/' );
        }

        protected function get_twitter_handle() {
            $link = $this->get_twitter_url();
            if ( $link ) {
                $bits = explode( '/', $link );
                return '@' . $bits[count($bits)-1];
            }
            return NULL;
        }

        protected function get_matching_sameAs_url( $site ) {
            $data = $this->get_user_schema( 'Organization' );
            if ( is_array( $data ) and !empty( $data['sameAs'] ) ) {
                foreach ( $data['sameAs'] as $link ) {
                    $link = trim( $link );
                    if ( wp_http_validate_url( $link ) && str_starts_with( $link, $site ) )
                        return $link;
                }
            }
            return NULL;
        }

        /************************************************************************
         *
         * Site schema customisation
         *
         ************************************************************************/

        protected $user_schema = NULL;

        function get_user_schema( $name ) {
            if ( NULL === $this->user_schema ) {
                $this->user_schema = [];

                $yaml = $this->get_option( 'schema' );
                if ( $yaml ) {
                    $data = yaml_parse( $yaml );
                    if ( is_array( $data ) )
                        $this->user_schema = $data;
                }
            }

            return (
                array_key_exists( $name, $this->user_schema )
                ? $this->user_schema[$name]
                : NULL
            );
        }

        // Helper for CLI interface
        function set_user_schema( $schema ) {
            $this->user_schema = $schema;
        }

        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        use PluginBaseSettings;

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'RS SEO Basics', 'rs-seo-basics' ),
                __( 'RS SEO Basics', 'rs-seo-basics' )
            );
        }

        function settings_define_sections_and_fields() {
            $section = $this->settings_add_section(
                'section_image',
                __( 'OpenGraph', 'rs-seo-basics' ),
                __( 'Customisations for OpenGraph markup', 'rs-seo-basics' )
            );

            $this->settings_add_field(
                'share_image', $section,
                __( 'Fallback share image', 'rs-seo-basics' ),
                'input',
                [
                    'size' => 60,
                    'help' => __( 'Enter URL, slug or attachment ID', 'rs-seo-basics' ),
                ]
            );

            $section = $this->settings_add_section(
                'section_schema',
                __( 'Schema.org', 'rs-seo-basics' ),
                __( 'Customisations for Schema.org markup', 'rs-seo-basics' )
            );

            $this->settings_add_field(
                'schema', $section,
                __( 'Schema.org YAML', 'rs-seo-basics' ),
                'textarea',
                [
                    'rows' => 32,
                ]
            );
        }

        // Called automatically by PluginBaseSettings
        function settings_sanitize_option( $setting ) {
            if ( !empty( $setting['schema'] ) ) {
                $errors = [];
                set_error_handler( function ( $errno, $errstr ) use ( &$errors ) {
                    $errors[] = $errstr;
                } );
                $data = yaml_parse( $setting['schema'] );
                restore_error_handler();

                foreach ( $errors as $error ) {
                    add_settings_error(
                        $this->option_name,
                        'rs-seo-basics-schema-yaml-error',
                        esc_html( $error )
                    );
                }
            }

            return $setting;
        }

        function do_admin_head_action_settings_css() {
            echo '<style type="text/css">textarea#rs_seo_basics_schema { font-family: monospace; }</style>';
        }


        /************************************************************************
         *
         *	User profiles -- add textarea for social media links (Schema sameAs)
         *
         ************************************************************************/

        protected $edit_user_profile_callback_hook = [
            'edit_user_profile',
            'show_user_profile',
            'user_new_form'
        ];

        function do_edit_user_profile_action( $user ) {
            printf( '<h2>%s</h2>', __( 'RS Seo Basics — Social media presence', 'rs-seo-basics' ) );

            $sameAs = get_user_meta( $user->ID, 'rs_seo_basics_sameAs', true ) ?? '';
            $field = 'rs_seo_basics_sameAs';

            printf( '<table class="form-table"><tr>'
                    . '<th><label for="%1$s">Social links</label></th>'
                    . '<td><textarea name="%1$s" id="%1$s" rows="5">%2$s</textarea>'
                    . '<p class="description">List links to your social media presences, one link per line.</p>'
                    . '</td></tr></table>',
                    esc_attr( $field ),
                    esc_html( $sameAs )
            );
        }

        protected $edit_user_profile_update_callback_hook = [
            'personal_options_update',
            'edit_user_profile_update',
            'user_register',
        ];

        function do_edit_user_profile_update_action( $user_id ) {
            if ( !current_user_can( 'edit_user', $user_id ) )
                return;

            if ( isset( $_REQUEST['rs_seo_basics_sameAs'] ) )
                update_user_meta( $user_id, 'rs_seo_basics_sameAs', $_REQUEST['rs_seo_basics_sameAs'] );
        }
    }

    SEO_Basics::instance();


    /************************************************************************
     *
     *	Basic CLI for testing
     *
     ************************************************************************/

    if ( defined( 'WP_CLI' ) && WP_CLI ) {


        /**
         * RS SEO basics test interface
         *
         * ## EXAMPLES
         *
         *     wp seo schema post ID
         *
         */

        class SEO_Basics_CLI {
            protected $plugin = NULL;
            function __construct() {
                $this->plugin = SEO_Basics::instance();
            }

            /**
             * Test schema generation
             *
             * ## OPTIONS
             *
             * <object>
             * : Test this kind of object
             * ---
             * default: post
             * options:
             *   - post
             *   - post-type
             *   - term
             *   - user
             *   - search
             *   - error
             * ---
             *
             * <id>
             * : ID of object to look up
             *
             * [--settings=<yaml-file>]
             * : Read settings from <yaml-file>
             *
             * ## EXAMPLES
             *
             *     wp seo schema post 42
             *
             */

            function schema(  $args, $assoc ) {
                $what = $args[0];
                $id = $args[1];

                if ( !empty( $assoc['settings'] ) ) {
                    $settings = yaml_parse_file( $assoc['settings'] );
                    if ( false === $settings ) {
                        \WP_CLI::error( "Failed to read {$assoc['settings']}");
                        return;
                    }

                    $this->plugin->set_user_schema( $settings );
                }

                switch ( $what ) {
                case 'post':
                    if ( !is_numeric( $id ) )
                        $id = url_to_postid( $id );
                    $post = get_post( (int)$id );
                    $schema = $this->plugin->schema_org( $post );
                    break;
                case 'post-type':
                    $post_type = get_post_type_object( $id );
                    $schema = $this->plugin->schema_org( $post_type );
                    break;
                case 'term':
                    $term = get_term( (int)$id );
                    $schema = $this->plugin->schema_org( $term );
                    break;
                case 'user':
                    $user = get_user_by( 'login', $id );
                    $schema = $this->plugin->schema_org( $user );
                    break;
                case 'search':
                    global $wp_query;
                    set_query_var( 's', $id );
                    $wp_query->is_search = true;
                    $schema = $this->plugin->schema_org( NULL );
                    break;
                }

                printf( "<script type=\"application/ld+json\">\n%s\n</script>\n", json_encode( $schema, JSON_PRETTY_PRINT ) );

            }

            /**
             * Test schema settings
             *
             * ## OPTIONS
             *
             * [<yaml-file>]
             * : Read this file.
             *
             * [--show]
             * : Display the YAML as-is
             */

            function settings(  $args, $assoc ) {
                if ( !empty( $args[0] ) )
                    $schema = file_get_contents( $args[0] );
                else
                    $schema = $this->plugin->get_option( 'schema' );

                $errors = [];
                set_error_handler( function ( $errno, $errstr ) use ( &$errors ) {
                    $errors[] = $errstr;
                } );
                $data = yaml_parse( $schema );
                restore_error_handler();

                if ( $errors ) {
                    foreach ( $errors as $error )
                        \WP_CLI::error( $error );
                } else {
                    if ( !empty( $assoc['show'] ) )
                        echo yaml_emit( $data );
                    else
                        \WP_CLI::success('No errors');
                }
            }
        }

        \WP_CLI::add_command( 'seo', new SEO_Basics_CLI() );
    }
} );
