<?php

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/************************************************************************
 *
 *	Open Graph meta data
 *
 ************************************************************************/

trait SEO_OpenGraph {

    // Find canonical and relative links for archive pages
    function link_rel() {
        $link_rel = [];

        $object = get_queried_object();

        if ( $object && in_array( get_class( $object ), [ 'WP_Post_Type', 'WP_Term', 'WP_User' ] ) ) {
            $page = get_query_var( 'paged', 1 );

            $url = get_pagenum_link( $page );
            if ( !empty( $url ) ) {
                $graph['og:url'] = $url;
                $link_rel['canonical'] = $url;
            }

            if  ( $page > 1 ) {
                $url = get_previous_posts_page_link();
                if ( !empty( $url ) )
                    $link_rel['prev'] = $url;
            }

            global $wp_query;

            $maxpage = $wp_query->max_num_pages;
            if ( $maxpage && $page < $maxpage ) {
                $url = get_next_posts_page_link();
                if ( !empty( $url ) )
                    $link_rel['next'] = $url;
            }
        }

        return $link_rel;
    }

    // Gather all the Open Graph data
    function opengraph() {
        $site_name = get_bloginfo( 'name' );

        // get_queried_object(): WP_Term|WP_Post_Type|WP_Post|WP_User|null

        $object = get_queried_object();

        if ( empty( $object ) ) {
            $object = (object)[];

            if ( is_search() )
                $graph = apply_filters( 'rs_seo_opengraph_search', [] );
            else if ( is_404() )
                $graph = apply_filters( 'rs_seo_opengraph_404', [] );
            else
                $graph = apply_filters( 'rs_seo_opengraph_unknown', [] );
        }
        else if ( is_a( $object, 'WP_Post' ) )
            $graph = apply_filters( 'rs_seo_opengraph_post', [], $object );
        else if ( is_a( $object, 'WP_Post_Type' ) )
            $graph = apply_filters( 'rs_seo_opengraph_post_type', [], $object );
        else if ( is_a( $object, 'WP_Term' ) )
            $graph = apply_filters( 'rs_seo_opengraph_term', [], $object );
        else if ( is_a( $object, 'WP_User' ) )
            $graph = apply_filters( 'rs_seo_opengraph_user', [], $object );

        if ( empty( $graph['og:title'] ) )
            $graph['og:title'] = $site_name . ' - ' . get_bloginfo( 'description' );

        if  ( $object )
            $graph = apply_filters( 'rs_seo_twitter', $graph, $object );

        return $graph;
    }

    // Basic starting point
    function opengraph_init() {
        return [
            'og:locale' => get_bloginfo( 'language' ),
            'og:site_name' => get_bloginfo( 'name' ),
            'og:type' => 'website',
            'og:logo' => get_site_icon_url(),
        ];
    }

    // Add the og:image tags
    function do_rs_seo_opengraph_image_filter( $graph, $image_id ) {
        if ( empty( $image_id ) ) {
            $fallback = $this->get_option( 'share_image' );

            if ( !empty( $fallback ) ) {
                if ( is_numeric( $fallback ) )
                    $image_id = (int)$fallback;
                else if ( $fallback == sanitize_title( $fallback ) ) {
                    $posts = get_posts( [
                        'post_type' => 'attachment',
                        'post_status' => 'inherit',
                        'name' => $fallback,
                        'posts_per_page' => 1,
                        'fields' => 'ids',
                    ] );

                    if ( !empty( $posts ) )
                        $image_id = $posts[0];
                } else {        // Assume URL
                    $image_id = attachment_url_to_postid( $fallback );
                }
            }

            if ( empty( $image_id ) )
                return $graph;
        }

        $url = wp_get_attachment_url( $image_id );
        $type = get_post_mime_type( $image_id );

        // Too many social media sites can't handle AVIF
        // This is not great but it'll show an image on social media
        if ( 'image/avif' === $type ) {
            $newtype = 'image/webp';
            $newext = '.webp';

            $newfile = get_post_meta( $image_id, 'rs_seo_opengraph_image', true );

            if ( empty( $newfile ) || !file_exists( $newfile ) ) {
                $file = get_attached_file( $image_id );

                $parts = pathinfo( $file );
                $newfile = $parts['dirname'] . '/' . $parts['filename'] . $newext;

                // Generate WEBP copy of image
                $editor = wp_get_image_editor( $file );

                if ( ! is_wp_error( $editor ) ) {
                    $saved = $editor->save( $newfile, $newtype );

                    if ( ! is_wp_error( $saved ) )
                        add_post_meta( $image_id, 'rs_seo_opengraph_image', $newfile, true );
                }
            }

            // If we have WEBP, then use it
            if ( file_exists( $newfile ) ) {
                // not strictly correct - but pathinfo() doesn't care
                $parts = pathinfo( $url );
                $url = $parts['dirname'] . '/' . $parts['filename'] . $newext;

                $type = $newtype;
            }
        }

        $graph['og:image'] = $url;
        $graph['og:image:type'] = $type;

        $meta = wp_get_attachment_metadata( $image_id );
        if ( is_array( $meta ) ) {
            $graph['og:image:width'] = $meta['width'];
            $graph['og:image:height'] = $meta['height'];
        }

        $alt = get_post_meta( $image_id, '_wp_attachment_image_alt', true );
        if ( !empty( $alt ) )
            $graph['og:image:alt'] = $alt;

        return $graph;
    }

    // Cleanup for AVIF special case
    function do_edit_attachment_action( $post_id ) {
        $newfile = get_post_meta( $image_id, 'rs_seo_opengraph_image', true );

        if ( !empty( $newfile ) && file_exists( $newfile ) )
            @unlink( $newfile );
    }

    function do_delete_attachment_action( $post_id, $post ) {
        $this->do_edit_attachment_action( $post_id );
    }

    // Grab featured image from first post -- as good as any
    function do_rs_seo_opengraph_guess_image_filter( $graph ) {
        global $wp_query;

        if ( have_posts() && !empty( $wp_query->posts[0] ) ) {
            $image_id = get_post_thumbnail_id( $wp_query->posts[0] );
            return apply_filters( 'rs_seo_opengraph_image', $graph, $image_id );
        }

        return $graph;
    }

    // Default filter for WP_Post
    function do_rs_seo_opengraph_post_filter( $graph, $post ) {
        if ( !empty( $graph ) )
            return $graph;

        $graph = $this->opengraph_init();
        $site_name = get_bloginfo( 'name' );

        if ( !is_front_page() )
            $graph['og:type'] =  'article';

        $graph['og:title'] = sprintf( '%s - %s', $post->post_title, $site_name );
        $graph['og:description'] = get_the_excerpt( $post );
        $graph['og:url'] = get_permalink( $post );

        $fb = $this->get_facebook_url();
        if ( $fb )
            $graph['article:publisher'] = $fb;

        $graph['article:modified_time'] = ( get_post_datetime( $post, 'modified' ) )->format( 'c' );
        $graph['article:published_time'] = ( get_post_datetime( $post, 'date' ) )->format( 'c' );

        $tags = get_the_tags( $post );
        if ( is_array( $tags ) )
            $graph['article:tag'] = wp_list_pluck( $tags, 'name' );

        if ( is_attachment() )
            $image_id = $post->ID;
        else
            $image_id = get_post_thumbnail_id( $post );

        return apply_filters( 'rs_seo_opengraph_image', $graph, $image_id );
    }

    // Default filter for WP_Post_Type
    function do_rs_seo_opengraph_post_type_filter( $graph, $post_type ) {
        if ( !empty( $graph ) )
            return $graph;

        $graph = $this->opengraph_init();
        $site_name = get_bloginfo( 'name' );

        $graph['og:title'] = sprintf( __( '%s Archive - %s', 'rs-seo-basics' ), $post_type->labels->name, $site_name );
        if ( !empty( $post_type->description ) )
            $graph['og:description'] = trim( $post_type->description );

        return apply_filters( 'rs_seo_opengraph_guess_image', $graph );
    }

    // Default filter for WP_Term
    function do_rs_seo_opengraph_term_filter( $graph, $term ) {
        if ( !empty( $graph ) )
            return $graph;

        $graph = $this->opengraph_init();
        $site_name = get_bloginfo( 'name' );

        $graph['og:title'] = sprintf( __( '%s Archive - %s', 'rs-seo-basics' ), $term->name, $site_name );
        if ( !empty( $term->description ) )
            $graph['og:description'] = trim( strip_tags( $term->description ) );

        return apply_filters( 'rs_seo_opengraph_guess_image', $graph );
    }

    // Default filter for WP_User
    function do_rs_seo_opengraph_user_filter( $graph, $user ) {
        if ( !empty( $graph ) )
            return $graph;

        $graph = $this->opengraph_init();
        $site_name = get_bloginfo( 'name' );

        $graph['og:type'] =  'profile';
        $graph['og:title'] = sprintf( __( '%s, author at %s', 'rs-seo-basics' ), $user->display_name, $site_name );
        if ( !empty( $user->description ) )
            $graph['og:description'] = trim( $user->description );

        $graph['profile:username'] =  $user->user_login;

        return apply_filters( 'rs_seo_opengraph_guess_image', $graph );
    }

    // Default filter for search result pages
    function do_rs_seo_opengraph_search_filter( $graph ) {
        if ( !empty( $graph ) )
            return $graph;

        $graph = $this->opengraph_init();
        $graph['og:title'] = sprintf( 'You searched for %s - %s', get_query_var( 's' ), get_bloginfo( 'name' ) );

        return $graph;
    }

    // Default filter for 404 pages
    function do_rs_seo_opengraph_404_filter( $graph ) {
        if ( !empty( $graph ) )
            return $graph;

        $graph = $this->opengraph_init();
        $graph['og:title'] = sprintf( 'Page not found - %s', get_bloginfo( 'name' ) );

        return $graph;
    }

    // Default filter for anything else
    function do_rs_seo_opengraph_unknown_filter( $graph ) {
        if ( !empty( $graph ) )
            return $graph;

        return $this->opengraph_init();
    }

    // Do the Nazi Twitter part
    function do_rs_seo_twitter_filter( $graph, $object ) {
        if ( empty( $graph['twitter:card'] ) )
            $graph['twitter:card'] = 'summary_large_image';

        if ( ! empty( $this->get_twitter_handle() ) )
            $graph['twitter:site'] = $this->get_twitter_handle();

        if ( is_a( $object, 'WP_Post' ) ) {
            // Reading time
            $wordcount = str_word_count( strip_tags( $object->post_content ) );
            $reading_time = intval( ($wordcount + 92) / 183 );
            if ( $reading_time > 0 ) {
                $graph['twitter:label1'] = 'Estimated reading time';
                $graph['twitter:data1'] = sprintf( '%d minutes', $reading_time );
            }
        }

        return $graph;
    }

    // Output the graph
    protected function opengraph_print_meta( $atts, $tag = 'meta' ) {
        printf( "\t<%s %s>\n", esc_attr( $tag ),
                join( ' ',
                      array_map(
                          fn( $k, $v ) => sprintf('%s="%s"', esc_attr( $k ), esc_attr( $v ) ),
                          array_keys( $atts ),
                          array_values( $atts )
                      )
                )
        );
    }
}
